@@ -7,7 +7,9 @@
 
 #include "ModelSource.h"
 
-#include <QDebug>
+#include <QMetaProperty>
+
+#include "charts_datasource_logging.h"
 
 ModelSource::ModelSource(QObject *parent)
     : ChartDataSource(parent)
@@ -20,6 +22,10 @@ ModelSource::ModelSource(QObject *parent)
 
 int ModelSource::role() const
 {
+    if (!m_model) {
+        return -1;
+    }
+
     if (m_role < 0 && !m_roleName.isEmpty()) {
         m_role = m_model->roleNames().key(m_roleName.toLatin1(), -1);
     }
@@ -49,16 +55,18 @@ bool ModelSource::indexColumns() const
 
 int ModelSource::itemCount() const
 {
-    if (!m_model)
+    if (!m_model) {
         return 0;
+    }
 
     return m_indexColumns ? m_model->columnCount() : m_model->rowCount();
 }
 
 QVariant ModelSource::item(int index) const
 {
-    if (!m_model)
-        return QVariant{};
+    if (!m_model) {
+        return {};
+    }
 
     // For certain model (QML ListModel for example), the roleNames() are more
     // dynamic and may only be valid when this method gets called. So try and
@@ -70,13 +78,13 @@ QVariant ModelSource::item(int index) const
 
         m_role = m_model->roleNames().key(m_roleName.toLatin1(), -1);
         if (m_role < 0) {
-            qWarning() << "ModelSource: Invalid role " << m_role << m_roleName;
+            qCWarning(DATASOURCE) << "ModelSource: Invalid role " << m_role << m_roleName;
             return QVariant{};
         }
     }
 
     if (!m_indexColumns && (m_column < 0 || m_column > m_model->columnCount())) {
-        qWarning() << "ModelSource: Invalid column" << m_column;
+        qCDebug(DATASOURCE) << "ModelSource: Invalid column" << m_column;
         return QVariant{};
     }
 
@@ -90,8 +98,13 @@ QVariant ModelSource::item(int index) const
 
 QVariant ModelSource::minimum() const
 {
-    if (itemCount() <= 0)
-        return QVariant{};
+    if (!m_model || itemCount() <= 0) {
+        return {};
+    }
+
+    if (m_minimum.isValid()) {
+        return m_minimum;
+    }
 
     auto minProperty = m_model->property("minimum");
     auto maxProperty = m_model->property("maximum");
@@ -101,15 +114,20 @@ QVariant ModelSource::minimum() const
 
     QVariant result = std::numeric_limits<float>::max();
     for (int i = 0; i < itemCount(); ++i) {
-        result = qMin(result, item(i));
+        result = std::min(result, item(i), variantCompare);
     }
     return result;
 }
 
 QVariant ModelSource::maximum() const
 {
-    if (itemCount() <= 0)
-        return QVariant{};
+    if (!m_model || itemCount() <= 0) {
+        return {};
+    }
+
+    if (m_maximum.isValid()) {
+        return m_maximum;
+    }
 
     auto minProperty = m_model->property("minimum");
     auto maxProperty = m_model->property("maximum");
@@ -119,7 +137,7 @@ QVariant ModelSource::maximum() const
 
     QVariant result = std::numeric_limits<float>::min();
     for (int i = 0; i < itemCount(); ++i) {
-        result = qMax(result, item(i));
+        result = std::max(result, item(i), variantCompare);
     }
     return result;
 }
@@ -180,6 +198,8 @@ void ModelSource::setModel(QAbstractItemModel *model)
 
     if (m_model) {
         m_model->disconnect(this);
+        m_minimum = QVariant{};
+        m_maximum = QVariant{};
     }
 
     m_model = model;
@@ -190,7 +210,49 @@ void ModelSource::setModel(QAbstractItemModel *model)
         connect(m_model, &QAbstractItemModel::modelReset, this, &ModelSource::dataChanged);
         connect(m_model, &QAbstractItemModel::dataChanged, this, &ModelSource::dataChanged);
         connect(m_model, &QAbstractItemModel::layoutChanged, this, &ModelSource::dataChanged);
+
+        connect(m_model, &QAbstractItemModel::destroyed, this, [this]() {
+            m_minimum = QVariant{};
+            m_maximum = QVariant{};
+            m_model = nullptr;
+        });
+
+        auto minimumIndex = m_model->metaObject()->indexOfProperty("minimum");
+        if (minimumIndex != -1) {
+            auto minimum = m_model->metaObject()->property(minimumIndex);
+            if (minimum.hasNotifySignal()) {
+                auto slot = metaObject()->method(metaObject()->indexOfSlot("onMinimumChanged()"));
+                connect(m_model, minimum.notifySignal(), this, slot);
+                m_minimum = minimum.read(m_model);
+            }
+        }
+
+        auto maximumIndex = m_model->metaObject()->indexOfProperty("maximum");
+        if (maximumIndex != -1) {
+            auto maximum = m_model->metaObject()->property(maximumIndex);
+            if (maximum.hasNotifySignal()) {
+                auto slot = metaObject()->method(metaObject()->indexOfSlot("onMaximumChanged()"));
+                connect(m_model, maximum.notifySignal(), this, slot);
+                m_maximum = maximum.read(m_model);
+            }
+        }
     }
 
     Q_EMIT modelChanged();
 }
+
+void ModelSource::onMinimumChanged()
+{
+    auto newMinimum = m_model->property("minimum");
+    if (newMinimum.isValid() && newMinimum != m_minimum) {
+        m_minimum = newMinimum;
+    }
+}
+
+void ModelSource::onMaximumChanged()
+{
+    auto newMaximum = m_model->property("maximum");
+    if (newMaximum.isValid() && newMaximum != m_maximum) {
+        m_maximum = newMaximum;
+    }
+}
